SystemOrganization addCategory: #Cart!
SystemOrganization addCategory: #'Cart-Test'!

Object subclass: #Cart
	instanceVariableNames: 'id books catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cart'!

Cart class
	instanceVariableNames: 'idCounter'!

Cart class
	instanceVariableNames: 'idCounter'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 15:58'!
bookMustBeInTheCatalogErrorMessage

	^'The book is not in the catalog'! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 15:52'!
empty

	^Cart new initializeWithId: self nextId! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 15:52'!
emptyWithCatalog: aCatalog

	^Cart new initializeWithId: self nextId andCatalog: aCatalog! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 11:40'!
nextId

	^idCounter := idCounter + 1! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 12:56'!
quantityMustNotBeLessThanOneErrorMessage

	^'Quantity must be at least one'! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 12:28'!
resetId

	^idCounter := 0! !

!Cart methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 21:41'!
addToCart: anIsbn withQuantity: aQuantity
	
	| totalQuantity |
	
	(catalog includesKey: anIsbn) ifFalse: [self signalBookMustBeInTheCatalog].
	
	(aQuantity < 1) ifTrue: [self signalQuantityMustNotBeLessThanOne].
	
	books add: anIsbn withOccurrences: aQuantity! !

!Cart methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 17:44'!
includes: aBook

	^books includes: aBook! !

!Cart methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 15:52'!
initializeWithId: anId andCatalog: aCatalog

	id := anId.
	books := Bag new.
	catalog := aCatalog! !

!Cart methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 12:19'!
isEmpty

	^books isEmpty! !

!Cart methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 16:56'!
quantityOf: aBook

	^books occurrencesOf: aBook! !

!Cart methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 15:58'!
signalBookMustBeInTheCatalog

	self error: self class bookMustBeInTheCatalogErrorMessage! !

!Cart methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/13/2012 12:55'!
signalQuantityMustNotBeLessThanOne

	self error: self class quantityMustNotBeLessThanOneErrorMessage! !

Object subclass: #Cashier
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cart'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/18/2012 16:30'!
cantCheckoutEmptyCartErrorMessage

	^'Can not checkout an empty cart'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/18/2012 17:10'!
creditCardMustNotBeExpiredErrorMessage

	^'The Credit Card is Expired'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/18/2012 18:23'!
signalExpirationFormatIsInvalidErrorMessage
	
	^'Credit Card expiration date format is invalid'! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/18/2012 18:39'!
checkOut: aCart withCreditCard: aCreditCard

	|date dateComplete|

	(aCart isEmpty) ifTrue: [self signalCanNotCheckoutAnEmptyCart].
	
	date := (aCreditCard value) value.
	
	dateComplete := '01/',date.
	
	(Date readFromString: dateComplete < Date today) 
					ifTrue: [self signalCreditCardIsExpired].
	
	((date matchesRegex: '[0][0-9]/\d\d\d\d') 
		or: (date matchesRegex: '[1][0-2]/\d\d\d\d'))
		ifFalse: [self signalExperationFormatIsInvalid]! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/18/2012 16:38'!
signalCanNotCheckoutAnEmptyCart

	^self error: self class cantCheckoutEmptyCartErrorMessage! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/18/2012 17:25'!
signalCreditCardIsExpired

	^self error: self class creditCardMustNotBeExpiredErrorMessage! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'Jamardo 6/18/2012 18:23'!
signalExperationFormatIsInvalid

	^self error: self class signalExpirationFormatIsInvalidErrorMessage! !

TestCase subclass: #CartTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cart-Test'!

!CartTest methodsFor: 'creation' stamp: 'Jamardo 6/13/2012 21:30'!
catalog

	
	| whiteBook blackBook blueBook aCatalog|
	
	whiteBook := Association key: 'ISBN-123' value: 5.
	blackBook := Association key: 'ISBN-456' value: 10.
	blueBook := Association key: 'ISBN-789' value: 15.
	
	aCatalog := Dictionary new. 
	aCatalog add: whiteBook.
	aCatalog add: blackBook.
	aCatalog add: blueBook.

	^aCatalog! !

!CartTest methodsFor: 'creation' stamp: 'Jamardo 6/18/2012 16:19'!
emptyCart
	
	| aCatalog |
	
	aCatalog := self catalog.
	
	^Cart emptyWithCatalog: aCatalog! !

!CartTest methodsFor: 'creation' stamp: 'Jamardo 6/18/2012 17:31'!
filledCart
	
	| aCatalog aCart whiteBook blackBook |
	
	whiteBook := 'ISBN-123'.
	blackBook := 'ISBN-456'.
	
	aCatalog := self catalog.	
	aCart := Cart emptyWithCatalog: aCatalog.
	
	aCart addToCart: whiteBook withQuantity: 1.
	aCart addToCart: blackBook withQuantity: 2.
	
	^aCart! !

!CartTest methodsFor: 'cartTests' stamp: 'Jamardo 6/18/2012 16:19'!
test01NewCartMustBeEmpty

	self assert: self emptyCart isEmpty! !

!CartTest methodsFor: 'cartTests' stamp: 'Jamardo 6/18/2012 16:19'!
test02AfterAddingABookToACartMustNotBeEmpty

	| aCart |
	
	aCart := self emptyCart.
	aCart addToCart: 'ISBN-123' withQuantity: 1.
	self deny: aCart isEmpty! !

!CartTest methodsFor: 'cartTests' stamp: 'Jamardo 6/18/2012 16:19'!
test03WhenAddingTwoBooksTheyMustBeInTheCart

	| whiteBook blackBook aCart |
	
	whiteBook := 'ISBN-123'.
	blackBook := 'ISBN-456'.
	
	aCart := self emptyCart.
	aCart addToCart: whiteBook withQuantity: 1.
	aCart addToCart: blackBook withQuantity: 1.
	
	self assert: (aCart includes: whiteBook).
	self assert: (aCart includes: blackBook).! !

!CartTest methodsFor: 'cartTests' stamp: 'Jamardo 6/18/2012 16:19'!
test04addingBooksMustThrowExceptionIfQuantityIsLessThanOne

	| aCart aBook |
	
	aCart := self emptyCart.
	aBook := 'ISBN-123'.
	[aCart addToCart: aBook withQuantity: -1.
	 self fail ]
		on: Error
		do: [ :anError | 
			self assert: anError messageText = Cart quantityMustNotBeLessThanOneErrorMessage. ]! !

!CartTest methodsFor: 'cartTests' stamp: 'Jamardo 6/13/2012 21:44'!
test05IsbnOfBookToAddMustBeInTheCatalog

	| redBook aCatalog aCart|
	redBook := 'ISBN-999'.
	aCatalog := self catalog.
	aCart := Cart emptyWithCatalog: aCatalog.

	[aCart addToCart: redBook withQuantity: 1.
	 self fail ]
		on: Error
		do: [ :anError | 
			self assert: anError messageText = Cart bookMustBeInTheCatalogErrorMessage. ]! !

!CartTest methodsFor: 'cartTests' stamp: 'Jamardo 6/18/2012 16:19'!
test06WhenAddingABookAlreadyInTheCartTheQuantityMustBeThePreviousQuantityPlusTheAddingQuantity

	| aCart whiteBook |
	
	aCart := self emptyCart.
	whiteBook := 'ISBN-123'.
	aCart addToCart: whiteBook withQuantity: 1.
	aCart addToCart: whiteBook withQuantity: 2.
	
	self assert: ((aCart quantityOf: whiteBook) = 3)! !

!CartTest methodsFor: 'cartTests' stamp: 'Jamardo 6/18/2012 16:55'!
test07checkoutingACartEmptyMustThrowException

	| emptyCart aCashier aCreditCard aNumber aDate anOwner |
	emptyCart := self emptyCart.
	aCashier := Cashier new.
	aCreditCard := Association key: anOwner value: (Association key: aNumber value: aDate).
	[aCashier checkOut: emptyCart withCreditCard: aCreditCard.
	 self fail]
		on: Error
		do: [ :anError | 
			self assert: anError messageText = Cashier cantCheckoutEmptyCartErrorMessage. ]! !

!CartTest methodsFor: 'cartTests' stamp: 'Jamardo 6/18/2012 18:24'!
test08CreditCardExpirationDateFormatMustBeValid
	
	| emptyCart aCashier aCreditCard aNumber aDate anOwner |
	emptyCart := self filledCart.
	aCashier := Cashier new.
	aDate := '1/2011'.
	anOwner := 'Don Quijote'.
	aNumber := 5678.
	
	aCreditCard := Association key: anOwner value: (Association key: aNumber value: aDate).
	[aCashier checkOut: emptyCart withCreditCard: aCreditCard.
	 self fail]
		on: Error
		do: [ :anError | 
			self assert: anError messageText = Cashier creditCardMustNotBeExpiredErrorMessage. ]! !

!CartTest methodsFor: 'cartTests' stamp: 'Jamardo 6/18/2012 18:41'!
test09CreditCardMustNotBeExpired

	| emptyCart aCashier aCreditCard aNumber aDate anOwner |
	emptyCart := self filledCart.
	aCashier := Cashier new.
	aDate := '10/2011'.
	anOwner := 'Don Quijote'.
	aNumber := 5678.
	aCreditCard := Association key: anOwner value: (Association key: aNumber value: aDate).
	[aCashier checkOut: emptyCart withCreditCard: aCreditCard.
	 self fail]
		on: Error
		do: [ :anError | 
			self assert: anError messageText = Cashier signalExpirationFormatIsInvalidErrorMessage. ]! !
